/*
 * $Id: pixmap.cpp,v 1.5 2002/07/17 07:29:59 inou Exp $
 */

#include "pixmap.h"
#include "colormap.h"
#include "canvas.h"
using namespace QFract;

// for debug
#include <iostream>
using namespace std;
#include <sys/time.h>

#include <qimage.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpen.h>
#include <qcolor.h>
#include <qwidget.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qapplication.h>
//
using namespace Qt;

Pixmap::Pixmap( Canvas* c, int w, int h, 
		double xl, double yt, double xr, double yb, 
		int m, int (*f)( Point, Parameter, int ),
		void (*pi)( Parameter ), Parameter p)
    : colormap()
{
  parent=c;
  image = new QImage( w, h, QImage::Format_RGB32 );
  region.setValue(xl,yt,xr,yb); max=m; iter=f; param.set(p); paraminit=pi;
  setDiff(); if (paraminit!=NULL) paraminit(param);
}

Pixmap::Pixmap( const Pixmap& d )
{
  parent=d.parent; region=d.region; iter = d.iter; param.set(d.param); 
  paraminit = d.paraminit; diff=d.diff;
  image = new QImage( *d.image );
 if (paraminit!=NULL) paraminit(param);
}

Point Pixmap::getPoint(int ix, int iy)
{ return Point( region.topLeft()+Point(diff*ix, -diff*iy) ); }

void Pixmap::draw()
{
  image->fill( black );
  
  struct timeval tv1,tv2;
  gettimeofday(&tv1, NULL);
  int stime=tv1.tv_sec*1000000+tv1.tv_usec;

  int i, k=1;
  for (int ix=0; ix<width(); ix++) {
    for (int iy=0; iy<height(); iy++) {
      Point z=getPoint(ix,iy);
      i=iter( z, param, max );
      if (i>=0) {
	image->setPixel( ix, iy, colormap.Rgb(i%NUM) );
      }
    }
    if ( (ix & 3) == 0 ) {
	gettimeofday(&tv2, NULL);
	int etime = tv2.tv_sec*1000000+tv2.tv_usec-stime;
	if ( (etime / 200000) >= k ) {
	    parent->setStatusBar( qApp->translate("QFract::Pixmap", "%1% done.").arg((ix*100)/width()) );
	    k = etime / 200000 +1;
	}
    }
  }
  /*
  gettimeofday(&tv2, NULL);
  cerr << "pixmap::draw(): "
       << tv2.tv_sec*1000000+tv2.tv_usec-stime << endl;
  */
  parent->setStatusBar(qApp->translate("QFract::Pixmap", "Done."));
}

void Pixmap::resize( int w, int h, bool redrawEnabled )
{ 
    if (image) delete image;
    image = new QImage( w, h, QImage::Format_RGB32 );
    setDiff();
    if (redrawEnabled) draw();
}

QPoint Pixmap::getPos(Point z)
{
  int ix= (int) ((z.x-region.xleft)/diff);
  int iy= (int) ((z.y-region.ytop)/(-diff));
  return QPoint(ix,iy);
}

void Pixmap::setDiff()
{
  double xd = region.width()/width();
  double yd = region.height()/height();
  diff =  (xd > -yd) ? xd : -yd;
}
